/* 
 *  SSE3DNow Copyright (c) Roy Longbottom 2002 - 2007
 *
 *  Version 1.1 Sept 2007 allocMem[] wrong for 8 GB RAM
 *
 *  SSE64 - 64 bit version where 3DNow and normal floating point
 *  instructions cannot be used
 *
 *  Can be compiled with Microsoft Visual C++ 6.0 Pro with Service 
 *  Pack 5 and Processor Pack, required to use the new instructions.
 *  These are used via assembly code or intrinsic functions. The
 *  Processor Pack includes a help file, procpack.chm, that describes
 *  all the intrinsics. For full details of Intel assembly instructions
 *  see intel.com:
 *
 *   IA-32 Intel Architecture Software Developer's Manual Volume 2: 
 *   Instruction Set Reference.
 *
 *  This program only uses the simpler floating point instructions. There are 
 *  others for SSE, SSE2 and 3DNow and non-floating point instructions. 
 *
 *  SSE and SSE2 use 128 bit registers in Single Instruction Multiple Data SIMD 
 *  mode. SSE is via 4 single precision 32 bit numbers and SSE2 via 2 double 
 *  precision 64 bit numbers. 3DNow uses two 32 bit numbers in MMX registers. 
 *  SSE instructions also need Operating System support to control their use, 
 *  so OS support has also to be determined.
 *
 *
 *  What the benchmark does
 *
 *  Calculates s=s+x[m]*y[m] and x[m]=x[m]+y[m] using SSE, SSE2 and
 *  3DNow instructions, as available, plus normal Single Precision
 *  FP instructions. Tests are carried out with different memory use
 *  from 4 Kbytes upwards.
 *
 *  Data is loaded in unrolled loops 64 bytes at a time to suit 64 byte bus 
 *  burst reads, leading to the number of instructions in a loop varying 
 *  between SSE, SSE2, 3DNow and SPFP. Obtaining high performance using 
 *  normal floating point instructions is rather difficult with multiples 
 *  of eight loads, calculations and stores. So this has been left for the 
 *  compiler to do. 
 *
 *  Modules
 *
 *  Config.cpp - via getDetails() identifies CPU type/speed, supported
 *             instruction extensions, memory size and OS version.
 *  This version via CPUasm.obj, asmtime.obj and asmtime.h - see
 *  NewSource.zip
 *
 *  SSE3DNow.rc - menus and dialog boxes, AboutDlgProc(), TestPanel(),
 *        RunPanel().
 *  SSE64Res.rc this version
 *
 *        TestPanel shows configuration details on starting and results
 *        at the end of testing. It has Run Tests and Exit buttons.
 *        RunPanel provides a speed summary as tests are running. 
 *
 *  SSE3DNow.h and SSE3DNow.ico
 *  SSE64.h this version
 *
 *  SSE3DNow.cpp
 *  SSE64.cpp this version with assembly code in SSECode.asm
 *
 *  Particularly with Windows XP, on running a benchmark which displays
 *  on-going results, the window for the latter can be covered by another
 *  window. To overcome this, all tests are run via the WinMain message
 *  processing loop using phase and jval variables, with jval controlling
 *  the size of memory to be tested.
 *
 *  Phase 1 via RUN_ALL button and MENU_RUNALL executes setUp() to 
 *  allocate maximum memory and open the results log file SSE3DNow.txt.
 *
 *  Phase 2 to 9 executes runTests() jval times, each for a particular
 *  runTime. The tests are only run if the appropriate instructions
 *  are available.
 *
 *  Phase 2 - SSE2One() assembly code
 *  Phase 3 - SSEOne() assembly code
 *  Phase 4 - One3DNow() assembly code
 *  Phase 5 - SPOne C code loop
 *  Phase 6 - SSE2Two() assembly code
 *  Phase 7 - SSETwo() assembly code
 *  Phase 8 - Two3DNow() assembly code
 *  Phase 9 - SPTwo C code loop
 *
 *  Current results are displayed at the start of runTests() and the log
 *  file is updated after phase 9.
 *
 *  Each do while (secs < runTime) loop executes checkTime()to increase
 *  the number of times each test is executed if needed (more testing
 *  less overheads).
 *
 *  start_time() and end_time() call high resolution timer to obtain
 *  running time secs.
 *  local_time() obtains date and clock time
 *  finish()completes the log file and releases allocated memory.
 *  displayWhatever() either closes TestPanel or closes RunPanel and
 *  opens TestPanel.
 *
 *  InitApp() determines maximum memory size to be used (25% of RAM ) but 
 *  this can be increased by commandLine parameters KBytes where test 
 *  runTime can also be changed from default of 0.1 seconds and autoRun
 *  set to run, log results and exit automatically.
 */
 
#define NAME "Test"
#define TITLE " Test Memory"
#define INITGUID

#include <windows.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <time.h>
#include <math.h>
#include <malloc.h>
#include <xmmintrin.h>
#include "SSE64.h"
#include "asmtime.h"

void SSE2One(); // kd
void SSEOne(); // ks
void SSE2Two(); // kd
void SSETwo(); // ks

double * xd;
double * yd;
float  * xs;
float  * ys;
double * dsum;
float  * ssum;
__int64   kd;
__int64   ks;
int   n1;


FILE    *outfile;


//  ************** PROTOPTYPES *****************

BOOL FAR PASCAL AboutDlgProc( HWND, UINT, WPARAM, LPARAM);
static BOOL InitApp(HINSTANCE, LPSTR, int);
LRESULT CALLBACK WindowProc(HWND, UINT, WPARAM, LPARAM);                  
static void CleanUpAndPostQuit(void);
void displayWhatever(HWND);

BOOL FAR PASCAL TestPanel( HWND, UINT, WPARAM, LPARAM);
BOOL FAR PASCAL RunPanel( HWND, UINT, WPARAM, LPARAM);

void local_time();
void end_time();
void checkTime();
BOOL runTests();
BOOL setUp();
BOOL finish();

// ************* GLOBAL VARIABLES **********
 
char   version[30] = "Version 1.1 for 64 bit OS";

char    timeday[30];
double  runTime = 0.1;

BOOL bQuit = FALSE;

HWND   mainHWND;
HWND   testHwnd = NULL;
HWND   runHwnd = NULL;
HINSTANCE MyInstance;  // HANDLE 
char    windowText[256];
char    commandLine[1024] = "";

short     keyPress;
BOOL  autoRun = FALSE;
BOOL  doTesting = FALSE;
char   writeMsg[256];
char   writeMsg1[256];
char   writeMsg2[256];
char   writeMsg3[256];
char   writeMsg4[256];
double results[30][9];
double maxmbps[8];

__int64 memUsed[100];
int  passesDone;
BOOL  runDone = FALSE;
__int64  ramKBused;
int  ramKB;
int runs = 10; // 2 MB;
__int64 vector[30];
DWORD allocMem[30];
RECT     winRect;
int     screenWide;
int     screenHigh;

int    maxrows = 13;
int    userows;
double hmfp, hmint;
double hmfp2, hmint2;
double sdfp, sdint;
double sdfp2, sdint2;
DWORD useMem;
int phase = 0;
int jval = 0;   


void displayWhatever(HWND hwnd)
{
    sprintf(windowText, " SSE, SSE2 Memory Test");
    SetWindowText(mainHWND, windowText);        
        
    if (doTesting)
    {
        if (testHwnd)   DestroyWindow(testHwnd);
    }
    else
    {    
        if (runHwnd)    DestroyWindow(runHwnd);
        testHwnd = CreateDialog( MyInstance,"DIALOG_8", hwnd, (DLGPROC)TestPanel);
    }  
} // displayWhatever


// WinMain - initialization, message loop

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                     LPSTR cmdline, int cmdshow)
{
    MSG         msg;
    
    hPrevInstance = hPrevInstance;

    if (!InitApp(hInstance, cmdline, cmdshow))
         return FALSE;

    GetMessage( &msg, NULL, 0, 0 );

    if (autoRun)
    {
         phase = 1;
    }        
    while(!bQuit)
    {
        if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
        {
            
           if (msg.message == WM_QUIT)
           {
                CleanUpAndPostQuit();
                break;
           }
   
           if( !GetMessage( &msg, NULL, 0, 0 ) ) return (int)msg.wParam;
           TranslateMessage(&msg);
           DispatchMessage(&msg);
        }
        else if (phase == 1)
        {
            if (!setUp())
            {
                CleanUpAndPostQuit();
                phase = 0;
            }
            phase = 2;
            jval = 0;
        }
        else if (phase > 1)
        {           
            if (!runTests())
            {
                CleanUpAndPostQuit();
                phase = -1;
            }
            phase = phase + 1;

            if (phase == 10)
            {
                phase = 2;
                jval = jval + 1;
                if (jval == runs)
                {
                    phase = 0;
                    finish();
                    if (autoRun)
                    {
                        CleanUpAndPostQuit();
                    }
                }
            }        
        }
        else
        {
           // make sure we go to sleep if we have nothing else to do
           WaitMessage();
        }
    }
    return (int)msg.wParam;
    
} // WinMain

// ***************************************************************************
//                    Initialization and object creation                     *
// ***************************************************************************

// InitApp
// Creates window

static BOOL
InitApp(HINSTANCE hInstance, LPSTR cmdline, int cmdshow)
{
    HWND hwnd;
    WNDCLASS wc;

    int   i, j;
    char *delims = {" ,"};
    char commands[20][256];
    char *p;
    int  KBytes;
    char buf[256] = "";
    HDC  hdc;
    int msecs = 0;
  
    
    // set up and registers the window class
    
    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = WindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = sizeof(DWORD);
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(hInstance, "GenericIcon");
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName = "GenericMenu";
    wc.lpszClassName = NAME;
    if (!RegisterClass(&wc))
        return FALSE;

    // Create the window

    hwnd =
        CreateWindow(
            NAME,
            TITLE,
            WS_OVERLAPPEDWINDOW | WS_HSCROLL | WS_VSCROLL,
            0,
            0,
            640,
            480,
            NULL, 
            NULL, 
            hInstance, 
            NULL 
        );

    if (!hwnd) return FALSE;

    mainHWND = hwnd;
    strcpy(commandLine, strupr(cmdline));

    hdc = GetDC(mainHWND);
    screenWide = GetDeviceCaps(hdc, HORZRES);
    screenHigh = GetDeviceCaps(hdc, VERTRES);   
    ReleaseDC(mainHWND, hdc);

    winRect.left = 0;
    winRect.top = 0;
    winRect.right = screenWide;
    winRect.bottom = screenHigh;

    getDetails();
    ramKB = ramsize * 1024;

    allocMem[0] = 2048;  // two arrays
    for (i=1; i<21; i++)
    {
        allocMem[i] = allocMem[i-1] * 2;
    }
    vector[0] = 240;        //    4 KB
    vector[1] = 496;        //    8 KB
    vector[2] = 1008;       //   16 KB 
    vector[3] = 2000;       //   32 KB
    vector[4] = 4000;       //   64 KB
    vector[5] = 8000;       //  128 KB
    vector[6] = 16000;      //  256 KB
    vector[7] = 32000;      //  512 KB
    vector[8] = 64000;      //    1 MB
    vector[9] = 128000;     //    2 MB
    vector[10] = 256000;    //    4 MB
    vector[11] = 512000;    //    8 MB
    vector[12] = 1024000;   //   16 MB
    vector[13] = 2048000;   //   32 MB
    vector[14] = 4096000;   //   64 MB
    vector[15] = 8192000;   //  128 MB
    vector[16] = 16384000;  //  256 MB
    vector[17] = 32768000;  //  512 MB
    vector[18] = 65536000;  // 1024 MB
    vector[19] = 131072000; // 2048 MB
    vector[20] = 262144000; // 4096 MB

    if (ramKB >   14000)  runs = 11;
    if (ramKB >   30000)  runs = 12;
    if (ramKB >   60000)  runs = 13;
    if (ramKB >  120000)  runs = 14;
    if (ramKB >  250000)  runs = 15;
    if (ramKB >  500000)  runs = 16;
    if (ramKB > 1000000)  runs = 17;
    if (ramKB > 2000000)  runs = 18;
    if (ramKB > 4000000)  runs = 19;
    if (ramKB > 8000000)  runs = 20;
    if (ramKB > 16000000) runs = 21;


    i = 0;
    p = strtok(commandLine, delims);
    while (p != NULL)
    {
        strcpy(commands[i], p);
        strcat(buf, commands[i]);
        strcat(buf, " ");
        i = i + 1;
        if (i > 4) break;
        p = strtok(NULL, delims);
    }
    if (i > 0)
    {
        strcpy(commandLine, buf);
    }
    else
    {
        strcpy(commandLine, "None");
    }
    autoRun = FALSE;
    KBytes = 0;
       
    for (j=0; j<i; j++)
    {
        switch(commands[j][0])
        {            
            case 'A': // Auto Run
             autoRun = TRUE;
             break;            

            case 'K': // KBytes
             if (j < i -1)
             {
                 sscanf(commands[j+1], "%d", &KBytes);
             }
             break;            
       
            case 'M': // millisecs
             if (j < i -1)
             {
                 sscanf(commands[j+1], "%d", &msecs);
             }
             break;
        }            
    }
    if (KBytes > 0)
    {
        runs = 1;
        j = 8;
        for (i=0; i<20; i++)
        {
            if (KBytes > j - 1) runs = runs + 1;
            j = j * 2; 
        }
    }
    if (msecs > 0) runTime = ((double) msecs) / 1000;

    ramKBused = vector[runs-1] * 16 / 1000;

    keyPress = GetAsyncKeyState(VK_ESCAPE);

    // Display the window
    
    ShowWindow(hwnd, cmdshow);    
    UpdateWindow(hwnd);

    if (!autoRun) displayWhatever(mainHWND);
    
    return TRUE;
    
}  // InitApp



// WindowProc - handle messages for the main application window


LRESULT CALLBACK  WindowProc(HWND hwnd, UINT msg,
                            WPARAM wparam, LPARAM lparam)                  
{
           
    switch( msg )
    {
    case WM_ACTIVATEAPP:
        break;
        
    case WM_INITMENUPOPUP:
        break;
        
    case WM_CREATE:
        break;

    case WM_DESTROY:
        CleanUpAndPostQuit();
        break;

    case WM_COMMAND:
        switch( LOWORD( wparam ) )
        {
        case MENU_ABOUT:
            if (testHwnd)   DestroyWindow(testHwnd);
            DialogBox( MyInstance,"AboutBox", hwnd, (DLGPROC)AboutDlgProc );
            displayWhatever(mainHWND);                                      
            break;

       case MENU_RUNALL:
            doTesting = TRUE;
            displayWhatever(mainHWND);                                      
            phase = 1;
            break;


        case MENU_DISPLAY:
            displayWhatever(mainHWND);                                      
            break;
        
        case MENU_EXIT:
            CleanUpAndPostQuit();
            break;
                
        }
        break;
   
        case WM_PAINT:
             break;
           
        case WM_KEYDOWN:
            break;

       case WM_ACTIVATE:
            break;
    } 
    return DefWindowProc( hwnd, msg, wparam, lparam );

} // WindowProc




// AboutDlgProc - processes messages for the about dialog.

BOOL FAR PASCAL AboutDlgProc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam )
{
    lparam = lparam;

    switch( msg ) {
    case WM_INITDIALOG:
        return( TRUE );

    case WM_COMMAND:
        if( LOWORD( wparam ) == IDOK ) {
            EndDialog( hwnd, TRUE );
            return( TRUE );
        }
        break;
    }
    return( FALSE );

} // AboutDlgProc



  
void local_time()
{
    time_t time_of_day;
    time_of_day = time( NULL ); 
    strcpy (timeday, ctime(&time_of_day)); // timeday = date and time
    return;
}




// CleanUpAndPostQuit
// Release all D3DRM objects, post a quit message and set the bQuit flag

void
CleanUpAndPostQuit(void)
{
    bQuit = TRUE;
    return;
}


BOOL FAR PASCAL RunPanel( HWND hdlg, UINT msg,
                                WPARAM wparam, LPARAM lparam )
{
    wparam = wparam;
    lparam = lparam;
    hdlg = hdlg;
    int  k;
    int FAR  tabs[10] = {1, 30, 60, 90, 120, 150, 180, 210, 240, 270};

     switch( msg )
     {
       case WM_INITDIALOG:
       
        SendDlgItemMessage(hdlg, RESULTS1, LB_SETTABSTOPS, (WPARAM) 10, (LPARAM) (int FAR*) tabs); 
        sprintf(writeMsg, "Maximum RAM size to be tested %d KB", ramKBused);
        SendDlgItemMessage(hdlg, NOTES1, LB_ADDSTRING, 0, (LPARAM) writeMsg);
        sprintf(writeMsg, "Flashing disk light suggest insufficient RAM and paging activity "
                      "to/from RAM and disk.");
        SendDlgItemMessage(hdlg, NOTES1, LB_ADDSTRING, 0, (LPARAM) writeMsg);
        sprintf(writeMsg, "Press ESC to cancel and exit program. It might take some time.");
        SendDlgItemMessage(hdlg, NOTES1, LB_ADDSTRING, 0, (LPARAM) writeMsg);
       
        if (runDone)
        {
            SendDlgItemMessage(hdlg, RESULTS1, LB_SETHORIZONTALEXTENT, 700, 0L);
            sprintf(writeMsg, " SSE Test %s by Roy Longbottom"
                              "%10.3f seconds minimum per test", version, runTime);
            SendDlgItemMessage(hdlg, RESULTS1, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            sprintf(writeMsg, " ");
            SendDlgItemMessage(hdlg, RESULTS1, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            sprintf(writeMsg, "Memory%c  s=s+x[m]*y[m]  %c%c%c x[m]=x[m]+y[m]", 9, 9, 9, 9); 

            SendDlgItemMessage(hdlg, RESULTS1, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            sprintf(writeMsg, "KBytes%c SSE2%c  SSE%c  3DN%c Sngl%c SSE2%c  SSE%c  3DN%c Sngl",
                                     9, 9, 9, 9, 9, 9, 9, 9);   
            SendDlgItemMessage(hdlg, RESULTS1, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            sprintf(writeMsg, "  Used%c MB/S%c MB/S%c  N/A%c  N/A%c MB/S%c MB/S%c  N/A%c  N/A%",
                                     9, 9, 9, 9, 9, 9, 9, 9);           
            SendDlgItemMessage(hdlg, RESULTS1, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            sprintf(writeMsg, " ");
            SendDlgItemMessage(hdlg, RESULTS1, LB_ADDSTRING, 0, (LPARAM) writeMsg);

            for (k=0; k<passesDone; k++)
            {
               sprintf(writeMsg, "%6d %c%7.0f %c%7.0f %c%7.0f %c%7.0f %c%7.0f %c%7.0f %c%7.0f %c%7.0f",  
                                memUsed[k]/1000, 9, results[k][0], 9, results[k][1], 9, results[k][2],
                                                 9, results[k][3], 9, results[k][4], 9, results[k][5],
                                                 9, results[k][6], 9, results[k][7]);
               SendDlgItemMessage(hdlg, RESULTS1, LB_ADDSTRING, 0, (LPARAM) writeMsg);
               SendDlgItemMessage(hdlg, RESULTS1, LB_SETCURSEL, k+6, 0L);
            }
        }
        
        return( TRUE );              
     }
     return( FALSE );
    
} // runPanel

BOOL FAR PASCAL TestPanel( HWND hdlg, UINT msg,
                                WPARAM wparam, LPARAM lparam )
{
    wparam = wparam;
    lparam = lparam;
    hdlg = hdlg;
    int  k;
    int FAR  tabs[10] = {1, 30, 60, 90, 120, 150, 180, 210, 240, 270};

     switch( msg )
     {
       case WM_INITDIALOG:
        SendDlgItemMessage(hdlg, RESULTS, LB_SETTABSTOPS, (WPARAM) 10, (LPARAM) (int FAR*) tabs); 
        sprintf(writeMsg, "Maximum RAM size to be tested %d KB at %10.3f seconds per test.",
                                    ramKBused, runTime);
        SendDlgItemMessage(hdlg, NOTES, LB_ADDSTRING, 0, (LPARAM) writeMsg);
        sprintf(writeMsg, "RAM size from GlobalMemoryStatus %d KB", ramKB);
        SendDlgItemMessage(hdlg, NOTES, LB_ADDSTRING, 0, (LPARAM) writeMsg);
        sprintf(writeMsg, "After running, results should be in file SSE64.txt and above");
        SendDlgItemMessage(hdlg, NOTES, LB_ADDSTRING, 0, (LPARAM) writeMsg);
       
        if (runDone)
        {
            SendDlgItemMessage(hdlg, RESULTS, LB_SETHORIZONTALEXTENT, 700, 0L);
            sprintf(writeMsg, " SSE Test %s by Roy Longbottom"
                              "%10.3f seconds minimum per test", version, runTime);
            SendDlgItemMessage(hdlg, RESULTS, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            sprintf(writeMsg, " ");
            SendDlgItemMessage(hdlg, RESULTS, LB_ADDSTRING, 0, (LPARAM) writeMsg);

            sprintf(writeMsg, "Memory%c  s=s+x[m]*y[m]  %c%c%c x[m]=x[m]+y[m]%c%c", 9, 9, 9, 9); 
            SendDlgItemMessage(hdlg, RESULTS, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            sprintf(writeMsg, "KBytes%c SSE2%c  SSE%c  3DN%c Sngl%c SSE2%c  SSE%c  3DN%c Sngl",
                                     9, 9, 9, 9, 9, 9, 9, 9);   
            SendDlgItemMessage(hdlg, RESULTS, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            sprintf(writeMsg, "  Used%c MB/S%c MB/S%c  N/A%c  N/A%c MB/S%c MB/S%c  N/A%c  N/A",
                                     9, 9, 9, 9, 9, 9, 9, 9);           
            SendDlgItemMessage(hdlg, RESULTS, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            sprintf(writeMsg, " ");
            SendDlgItemMessage(hdlg, RESULTS, LB_ADDSTRING, 0, (LPARAM) writeMsg);

            for (k=0; k<passesDone; k++)
            {
               sprintf(writeMsg, "%6d %c%7.0f %c%7.0f %c%7.0f %c%7.0f %c%7.0f %c%7.0f %c%7.0f %c%7.0f",  
                                memUsed[k]/1000, 9, results[k][0], 9, results[k][1], 9, results[k][2],
                                                 9, results[k][3], 9, results[k][4], 9, results[k][5],
                                                 9, results[k][6], 9, results[k][7]);
               SendDlgItemMessage(hdlg, RESULTS, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            }
            sprintf(writeMsg, " ");
            SendDlgItemMessage(hdlg, RESULTS, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            sprintf(writeMsg, " Maximum");
            SendDlgItemMessage(hdlg, RESULTS, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            sprintf(writeMsg, " Mflops%c%7.0f %c%7.0f %c%7.0f %c%7.0f %c%7.0f %c%7.0f %c%7.0f %c%7.0f",
                    9, maxmbps[0], 9, maxmbps[1], 9, maxmbps[2], 9, maxmbps[3],
                    9, maxmbps[4], 9, maxmbps[5], 9, maxmbps[6], 9, maxmbps[7]);
            SendDlgItemMessage(hdlg, RESULTS, LB_ADDSTRING, 0, (LPARAM) writeMsg);       
            SendDlgItemMessage(hdlg, RESULTS, LB_SETCURSEL, passesDone+11, 0L);

        }
        else
        {
            strcpy(writeMsg, "Batch Command ");
            strncat(writeMsg, commandLine, 200);
            SendDlgItemMessage(hdlg, RESULTS, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            SendDlgItemMessage(hdlg, RESULTS, LB_ADDSTRING, 0, (LPARAM) writeMsg1);
            SendDlgItemMessage(hdlg, RESULTS, LB_ADDSTRING, 0, (LPARAM) writeMsg2);
            SendDlgItemMessage(hdlg, RESULTS, LB_ADDSTRING, 0, (LPARAM) writeMsg3);
            SendDlgItemMessage(hdlg, RESULTS, LB_ADDSTRING, 0, (LPARAM) writeMsg4);
                }
        return( TRUE );
                    
       case WM_COMMAND:

        if( HIWORD( wparam ) == BN_CLICKED)
        {
            switch (LOWORD (wparam))
            {
                case RUN_ALL:
                  PostMessage(mainHWND, WM_COMMAND, MENU_RUNALL, 0);
                 break;

                case EXIT_BUTTON:
                 PostMessage(mainHWND, WM_COMMAND, MENU_EXIT, 0);
                 break;
            }
        }
        return( TRUE );         
     }
     return( FALSE );
    
} // TestPanel



BOOL setUp()
{
    int i, j;
    HDC  hdc;
    
    for (i=0; i<30; i++)
    {
        for (j=0; j<9; j++)
        {
            results[i][j] = 0;
        }
    }
    useMem = allocMem[runs - 1]; // vector[runs - 1] * 8 + 8;    

    hdc = GetDC(mainHWND);
    sprintf(writeMsg, "                          Allocating Memory  ");
    TextOut(hdc, 100, 100, writeMsg, lstrlen(writeMsg));    
    ReleaseDC(mainHWND, hdc);

    outfile = fopen("SSE64.txt","a+");
    if (outfile == NULL)
    {
        sprintf(writeMsg, "Cannot open results file");
        if (!autoRun) MessageBox(NULL, writeMsg, "ERROR WILL EXIT", MB_ICONWARNING | MB_OK);
        CleanUpAndPostQuit();
        return FALSE;
    }

    local_time();
    fprintf(outfile, "\n  ***************************************************************");
    fprintf(outfile, "\n  SSE and SSE2 Memory Reading Speed Test %s\n", version);      
    fprintf(outfile, "\n             Copyright (C) Roy Longbottom 2007\n");    
    fprintf(outfile, "\n %10.3f seconds per test, Start %s\n", runTime, timeday);
    fprintf(outfile, " MB/S is read speed. Multiply second set by 1.5 for read/write speed\n\n");
    fprintf(outfile, "  Memory          s=s+x[m]*y[m]               x[m]=x[m]+y[m]\n"); 
    fflush(outfile);

/*
  For the fastest SSE instructions used, all data loaded or stored MUST be 
  aligned on 16 byte address boundaries, otherwise the program hangs.

  useMem+65536 avoids severe performance problems with P4 CPU when 2 arrays
  are aligned exactly on MB boundaries
*/

    xd = (double *)_aligned_malloc(useMem+65536, 16);
    if (xd == NULL)
    {
        sprintf(writeMsg, "Cannot allocate memory");
        if (!autoRun) MessageBox(NULL, writeMsg, "ERROR WILL EXIT", MB_ICONWARNING | MB_OK);
        fprintf(outfile,"Cannot allocate memory\n\n");
        fflush(outfile);
        fclose(outfile);
        CleanUpAndPostQuit();
        return FALSE;
    }        
    yd = (double *)_aligned_malloc(useMem, 16);
    if (yd == NULL)
    {
        sprintf(writeMsg, "Cannot allocate memory");
        if (!autoRun) MessageBox(NULL, writeMsg, "ERROR WILL EXIT", MB_ICONWARNING | MB_OK);
        fprintf(outfile,"Cannot allocate memory\n\n");
        fflush(outfile);
        fclose(outfile);
        _aligned_free(xd);
        CleanUpAndPostQuit();
        return FALSE;
    }        
    dsum = (double *)_aligned_malloc(64, 16);
    if (dsum == NULL)
    {
        sprintf(writeMsg, "Cannot allocate memory");
        if (!autoRun) MessageBox(NULL, writeMsg, "ERROR WILL EXIT", MB_ICONWARNING | MB_OK);
        fprintf(outfile,"Cannot allocate memory\n\n");
        fflush(outfile);
        fclose(outfile);
        _aligned_free(xd);
        _aligned_free(yd);
        CleanUpAndPostQuit();
        return FALSE;
    }        
    xs   = (float  * )xd;
    ys   = (float  * )yd;
    ssum = (float  * )dsum;

    fprintf(outfile, "  KBytes    SSE2    SSE  3DNow   Sngl   SSE2    SSE  3DNow   Sngl\n");   
    fflush(outfile);

    runDone = TRUE;    
    passesDone = 0;

    sprintf(writeMsg, "                           Running Tests                    ");
    hdc = GetDC(mainHWND);
    TextOut(hdc, 100, 100, writeMsg, lstrlen(writeMsg));    
    ReleaseDC(mainHWND, hdc);
    GdiFlush();

    fprintf(outfile, "   Used     MB/S   MB/S    N/A    N/A   MB/S   MB/S    N/A    N/A\n\n");
    fflush(outfile);

    for (i=0; i<8; i++)
    {
        maxmbps[i] = 0;
    }
    return TRUE;

} // setUp


BOOL runTests()
{

    float  sums;
    float  ones = 1.0;
    float  zeros = 0;
    int i, m, j;
    int nn; 
    __int64 mem; 
    double memBytes;
    double BytesPerFLOP;
    double sumd;
    
    double oned = 1.0;
    double zerod = 0;
        
    if (runHwnd)    DestroyWindow(runHwnd);
    runHwnd = CreateDialog( MyInstance,"DIALOG_9", mainHWND, (DLGPROC)RunPanel);

    j = jval;
    {
        kd = vector[j];
        nn = (int)(6400000 / kd);  // min 240
        if (nn < 1) nn = 1;
        
        ks = kd * 2;
        
        mem = kd * 16;
        memUsed[j] = mem + 200;
        
        // SSE2One *******************************  

        if (phase == 2)
        {
            n1 = nn;
      
            do
            {
                sumd = 0;
                dsum[0] = 0;
                dsum[1] = 0;
                for (m=0; m<kd+6; m++)
                {
                      xd[m] = oned;
                      yd[m] = oned;
                }          
        
                start_time();
                for (i=0; i<n1; i++)
                {
                     SSE2One();
                }
                end_time();
                checkTime();
            }
            while (secs < runTime);
    
            sumd = dsum[0] + dsum[1];
            memBytes = 16;
            results[j][0] = (double)kd * (double)n1 * memBytes / 1000000/ secs;
            BytesPerFLOP = 8;            
            if (results[j][0] / BytesPerFLOP > maxmbps[0]) maxmbps[0] = results[j][0] / BytesPerFLOP;
        }
    
        //  SSEOne ******************************* 

        if (phase == 3)
        {
            n1 = nn;
            do
            {
                ssum[0] = 0;
                ssum[1] = 0;
                ssum[2] = 0;
                ssum[3] = 0;
                for (m=0; m<ks+12; m++)
                {
                    xs[m] = ones;
                    ys[m] = ones;
                }          
                start_time();
                for (i=0; i<n1; i++)
                {
                   SSEOne();
                }
                end_time();
                checkTime();            
            }
            while (secs < runTime);
    
            sums = ssum[0] + ssum[1] + ssum[2] + ssum[3];        
            memBytes = 8;
            results[j][1] = (double)ks * (double)n1 * memBytes / 1000000 / secs;
            BytesPerFLOP = 4;            
            if (results[j][1] / BytesPerFLOP > maxmbps[1]) maxmbps[1] = results[j][1] / BytesPerFLOP;
        }


        if (phase == 4) // was 3DNowOne
        {
        }


        if (phase == 5) // was SPOne
        {
        }
    
        //  SSE2Two ******************************* 
        
        if (phase == 6)
        {
            n1 = nn;
            do
              {
                for (m=0; m<kd; m++)
                {
                    xd[m] = zerod;
                    yd[m] = oned;
                }          
                start_time();
                for (i=0; i<n1; i++)
                {
                   SSE2Two();
                }
                end_time();
                checkTime();
              }
            while (secs < runTime);
        
            sumd = xd[1];
            memBytes = 16;
            results[j][4] = (double)kd * (double)n1 * memBytes / 1000000 / secs;
            BytesPerFLOP = 16;            
            if (results[j][4] / BytesPerFLOP > maxmbps[4]) maxmbps[4] = results[j][4] / BytesPerFLOP;
        }
    
        //  SSETwo ******************************* 

        if (phase == 7)
        {
            n1 = nn;
            do
              {
                for (m=0; m<ks; m++)
                  {
                      xs[m] = zeros;
                      ys[m] = ones;
                  }          
                start_time();
                for (i=0; i<n1; i++)
                  {
                     SSETwo();
                  }
                end_time();
                checkTime();
              }
            while (secs < runTime);
                
            sums = xs[1];
            memBytes = 8;
            results[j][5] = (double)ks * (double)n1 * memBytes / 1000000 / secs;
            BytesPerFLOP = 8;            
            if (results[j][5] / BytesPerFLOP > maxmbps[5]) maxmbps[5] = results[j][5] / BytesPerFLOP;
        }

        if (phase == 8) // Was 3DNowTwo
        {
        }
 
        //  SPTwo ******************************* 

        if (phase == 9) // Was SPTwo
        {
            fprintf(outfile, "%7d  %7.0f%7.0f%7.0f", memUsed[j] /1000,
                                                  results[j][0], results[j][1], results[j][2]);
            fprintf(outfile, "%7.0f%7.0f%7.0f",   results[j][3], results[j][4], results[j][5]);
            fprintf(outfile, "%7.0f%7.0f\n",      results[j][6], results[j][7]);
    
            fflush(outfile);
        }
        passesDone = j + 1;
        if (runHwnd)    DestroyWindow(runHwnd);
    
        if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
        {
            fprintf(outfile, "\n\n\n");
            _aligned_free(xd);
            _aligned_free(yd);
            _aligned_free(dsum);
            fflush(outfile);
            fclose(outfile);
            CleanUpAndPostQuit();
            return FALSE;
        }
    }
    return TRUE;
    
} // runTests

BOOL finish()
{
    int i;
    HDC  hdc;
    
    fprintf(outfile, "\n");
    fprintf(outfile, "            SSE2    SSE  3DNow   Norm   SSE2    SSE  3DNow   Norm\n");   
    fprintf(outfile, " Maximum      DP     SP    N/A    N/A     DP     SP    N/A    N/A\n");
    fprintf(outfile, "  MFLOPS ");
    for (i=0; i<8; i++)
    {
        fprintf(outfile, "%7.0f", maxmbps[i]);
    }
    fprintf(outfile, "\n");
    

    local_time();
    if (autoRun) DestroyWindow(runHwnd);

    fprintf(outfile, "\n                End of test %s", timeday);        
    fprintf(outfile,"\n");

    getDetails();
    for (i=0; i<9; i++)
    {
        fprintf(outfile, "%s\n", configdata[i]);
    }
    fprintf(outfile, "\n");

    fflush(outfile);
    fclose(outfile);
    _aligned_free(xd);
    _aligned_free(yd);
    _aligned_free(dsum);
    doTesting = FALSE;
    if (!autoRun) displayWhatever(mainHWND);                                      

    hdc = GetDC(mainHWND);
    Rectangle(hdc, winRect.left, winRect.top,
                   winRect.right, winRect.bottom);
    ReleaseDC(mainHWND, hdc);

    return TRUE;
    
} // finish


void checkTime()
{
    if (secs < runTime)
    {
          if (secs < runTime / 8.0)
          {
                n1 = n1 * 10;
          }
          else
          {
                n1 = (int)(runTime * 1.25 / secs * (double)n1+1);
          }
    }
// sprintf(writeMsg, "n1 %d secs %8.5f runTime %8.5f new n1 %d", nn1, secs, runTime, n1);
// MessageBox(NULL, writeMsg, "HERE", MB_ICONWARNING | MB_OK);
}


         
